#include <stdio.h>
#include "wwwsys.h"
#include "HTUtils.h"
#include "HTList.h"
#include <check.h>

#define EXIT_SUCCESS 0
#define EXIT_FAILURE 1

struct test_object {
    int number;
};

/* Exercises the public subroutines in Library/src/HTList.c */

START_TEST (test_newList)
{
    HTList *myList;

    myList = HTList_new();
    fail_if(myList == NULL);
    fail_unless(myList->object == NULL);
    fail_unless(myList->next == NULL);
    fail_unless(HTList_delete(myList));
}
END_TEST

START_TEST (test_addRemoveCount)
{
    HTList *myList;
    struct test_object one, two, three;

    one.number = 1;
    two.number = 2;
    three.number = 3;

    myList = HTList_new();
    fail_unless(HTList_addObject(myList, &one)); /* List = 1 */
    fail_unless(HTList_addObject(myList, &two)); /* List = 1 2 */
    fail_unless(HTList_count(myList) == 2);
    fail_unless(HTList_addObject(myList, &three)); /* List = 1 2 3 */ 
    fail_unless(HTList_count(myList) == 3);
    fail_unless(HTList_removeObject(myList, &two)); /* List = 1 3 */
    fail_unless(HTList_count(myList) == 2);
    fail_if(HTList_removeObject(myList, &two)); /* List = 1 3 */
    fail_unless(HTList_removeObject(myList, &three)); /* List = 1 */
    fail_unless(HTList_count(myList) == 1);
    fail_unless(HTList_delete(myList));
}
END_TEST

START_TEST (test_firstLast)
{
    HTList *myList;
    struct test_object one, two, three;
    struct test_object *tp;

    one.number = 1;
    two.number = 2;
    three.number = 3;

    myList = HTList_new();
    fail_unless(HTList_addObject(myList, &one)); /* List = 1 */
    fail_unless(HTList_addObject(myList, &two)); /* List = 1 2 */
    fail_unless(HTList_addObject(myList, &three)); /* List = 1 2 3 */ 
    tp = (struct test_object *) (HTList_firstObject(myList));
    fail_unless(tp->number == 1);
    tp = (struct test_object *) (HTList_objectAt(myList, 1));
    fail_unless(tp->number == 2);
    tp = (struct test_object *) (HTList_removeLastObject(myList)); /* 1 2 */
    fail_unless(tp->number == 3);
    tp = (struct test_object *) (HTList_removeFirstObject(myList)); /* 2 */
    fail_unless(tp->number == 1);
    fail_unless(HTList_count(myList) == 1);
    tp = (struct test_object *) (HTList_removeLastObject(myList));
    fail_unless(tp->number == 2);
    fail_unless(HTList_removeLastObject(myList) == NULL);
    fail_unless(HTList_delete(myList));
}
END_TEST

START_TEST (test_appendList)
{
    HTList *myList, *listNode;
    struct test_object one, two, three;
    struct test_object *tp;

    one.number = 1;
    two.number = 2;
    three.number = 3;

    myList = HTList_new();
    fail_unless(HTList_addObject(myList, &one)); /* List = 1 */
    fail_unless(HTList_appendObject(myList, &two)); /* List = 2 1 */
    listNode = HTList_addList(myList, &three); /* List = 2 1 3 */
    fail_unless(HTList_count(listNode) == 2);
    tp = (struct test_object *) (HTList_objectAt(listNode, 0));
    fail_unless(tp->number == 1);
    tp = (struct test_object *) (HTList_firstObject(listNode));
    fail_unless(tp->number == 2);

    listNode = HTList_elementOf(myList, &one, NULL);
    fail_unless(HTList_count(listNode) == 1);
    fail_unless(HTList_indexOfElement(myList, listNode) == 1);
    fail_unless(HTList_indexOf(myList, &two) == 2);

    fail_unless(HTList_delete(myList));
}
END_TEST

START_TEST (test_countDups)
{
    HTList *myList;
    struct test_object one, two, three;

    one.number = 1;
    two.number = 2;
    three.number = 3;

    myList = HTList_new();
    fail_unless(HTList_addObject(myList, &one)); /* List = 1 */
    fail_unless(HTList_appendObject(myList, &two)); /* List = 2 1 */
    fail_unless(HTList_appendObject(myList, &two)); /* List = 2 2 1 */
    fail_unless(HTList_addObject(myList, &two)); /* List = 2 2 1 2 */
    fail_unless(HTList_addObject(myList, &three)); /* List = 2 2 1 2 3 */
    fail_unless(HTList_count(myList) == 5);
    fail_unless(HTList_removeObjectAll(myList, &one)); /* List = 2 2 2 3 */
    fail_unless(HTList_count(myList) == 4);
    fail_unless(HTList_removeObjectAll(myList, &two)); /* List = 3 */
    fail_unless(HTList_count(myList) == 1);

    fail_unless(HTList_delete(myList));
}
END_TEST

Suite *list_suite(void)
{
   Suite *s = suite_create("Lists");
   TCase *tc_core = tcase_create("Core");

   suite_add_tcase(s, tc_core);
   tcase_add_test(tc_core, test_newList);
   tcase_add_test(tc_core, test_addRemoveCount);
   tcase_add_test(tc_core, test_firstLast);
   tcase_add_test(tc_core, test_appendList);
   tcase_add_test(tc_core, test_countDups);

   return s;
}

int main(void)
{
    int nf;
    Suite *s = list_suite();
    SRunner *sr = srunner_create(s);
    srunner_run_all(sr, CK_NORMAL);
    nf = srunner_ntests_failed(sr);
    srunner_free(sr);
    return (nf == 0) ?EXIT_SUCCESS: EXIT_FAILURE;
}
